@@ -25,7 +25,7 @@
   (= (float-sign-bit float) 1))
 (declaim (inline flonum-minus-zero-p))
 (defun flonum-minus-zero-p (flonum)
-  (or (eq flonum $-0.0f0) (eq flonum $-0.0d0)))
+  (or (eq flonum -0.0f0) (eq flonum -0.0d0)))
 
 (defun pick-result-format (&rest args)
   (flet ((target-num-fmt (num)
@@ -45,40 +45,18 @@
             (setq result-fmt arg-fmt result-contagion arg-contagion)))))))
 
 (defun pick-float-result-format (&rest args)
-  (flet ((target-num-fmt (num)
-           (cond ((rationalp num) 'single-float)
-                 ((floatp num) (type-of num))
-                 (t (error "What? ~S" num)))))
-    (let* ((result-fmt 'single-float)
-           (result-contagion 0))
-      (dolist (arg args result-fmt)
-        (let* ((arg-fmt (target-num-fmt arg))
-               ;; This is inadequate for complex numbers,
-               ;; but we don't need them.
-               (arg-contagion
-                (position arg-fmt
-                          '(short-float single-float double-float long-float))))
-          (when (cl:> arg-contagion result-contagion)
-            (setq result-fmt arg-fmt result-contagion arg-contagion)))))))
-
-(defun rational (x)
-  (cond
-    ((rationalp x) x)
-    ((float-infinity-or-nan-p x)
-     (error "Can't convert Inf or NaN to rational."))
-    (t (with-memoized-math-op (rational x)
-         (multiple-value-bind (significand exponent sign)
-             (integer-decode-float x)
-           (cl:* sign significand (cl:expt 2 exponent)))))))
+  (let ((format (apply #'pick-result-format args)))
+    (if (eq format 'rational)
+        'single-float
+        format)))
 
 (defun rationalize (x)
   (if (rationalp x)
       x
-      (with-memoized-math-op (rationalize x)
-        (let ((rational (rational x)))
-          (if (integerp rational)
-              rational
-              (error "Won't do (RATIONALIZE ~S) due to possible precision loss" x))))))
+      (let ((rational (rational x)))
+        (if (integerp rational)
+            rational
+            (error "Won't do (RATIONALIZE ~S) due to possible precision loss" x)))))
 
 (defun xfloat-coerce (object type)
   (declare (number object))
@@ -97,7 +75,7 @@
            (error "Can't COERCE ~S ~S" object type))
           ((and (flonum-minus-zero-p object)
                 (member type '(double-float single-float)))
-           (ecase type (single-float $-0.0f0) (double-float $-0.0d0)))
+           (ecase type (single-float -0.0f0) (double-float -0.0d0)))
           ((and (floatp object)
                 (float-infinity-p object))
            (ecase type
@@ -117,20 +95,16 @@
              (flonum-from-rational source-value actual-type))))))
 
 (defun xfloat-abs (x)
-  (with-memoized-math-op (abs x)
-    (if (float-sign-bit-set-p x)
-        (sb-xc:- x)
-        x)))
+  (if (float-sign-bit-set-p x)
+      (sb-xc:- x)
+      x))
 
 ;;; Signum should return -0 of the correct type for -0 input.
 ;;; We don't need it currently.
 (defun xfloat-signum (x)
   (if (zerop x)
       x
-      (coerce (if (float-sign-bit-set-p x)
-                  -1
-                  1)
-              (type-of x))))
+      (float-sign x)))
 
 (macrolet ((define (name float-fun)
              (declare (ignore name))
@@ -140,24 +114,14 @@
   (define mod xfloat-mod)
   (define rem xfloat-rem))
 
-(defun float (number &optional (prototype nil prototypep))
-  (validate-args number prototype)
-  (with-memoized-math-op (float (cons number (if prototypep (list prototype))))
-    (let* ((format
-             (if (not prototypep) 'single-float (type-of prototype))))
-      (flonum-from-rational (rational number) format))))
-
 (macrolet ((define (name float-fun)
              (let ((clname (intern (string name) "CL")))
                `(defun ,float-fun (number divisor)
-                  (let ((type (if (or (typep number 'double-float)
-                                      (typep divisor 'double-float))
-                                  'double-float
-                                  'single-float)))
+                  (let ((type (pick-float-result-format number divisor)))
                     (with-memoized-math-op (,name (list number divisor))
                       (if (flonum-minus-zero-p number)
                           (values 0
-                                  (coerce (if (< divisor 0) 0 $-0.0) type))
+                                  (coerce (if (< divisor 0) 0 -0.0) type))
                           (multiple-value-bind (q r)
                               (,clname (rational number) (rational divisor))
                             (values q (flonum-from-rational r type))))))))))
@@ -175,22 +139,20 @@
     (float (if (float-sign-bit-set-p thing) -1 1))))
 
 (macrolet ((define (name clname)
-             `(progn
-                (defun ,name (number &optional (divisor 1 divisorp))
-                  (let ((type (if (or (typep number 'double-float)
-                                      (typep divisor 'double-float))
-                                  'double-float
-                                  'single-float))
-                        (format (pick-result-format number divisor)))
-                    (with-memoized-math-op (,name (list* number (and divisorp (list divisor))))
-                      (multiple-value-bind (q r)
-                          (,clname (rational number) (rational divisor))
-                        (let ((remainder (if (eql format 'rational) r (flonum-from-rational r format))))
-                          (if (cl:= q 0)
-                              (values (coerce (if (cl:= (sgn number) (sgn divisor)) 0 $-0.0)
-                                              type)
-                                      remainder)
-                              (values (flonum-from-rational q type) remainder))))))))))
+             `(defun ,name (number &optional (divisor 1 divisorp))
+                (with-memoized-math-op (,name (if divisorp (list number divisor) number))
+                  (multiple-value-bind (q r)
+                      (,clname (rational number) (rational divisor))
+                    (let* ((type (pick-float-result-format number divisor))
+                           (format (pick-result-format number divisor))
+                           (remainder (if (eql format 'rational)
+                                          r
+                                          (flonum-from-rational r format))))
+                      (if (cl:= q 0)
+                          (values (coerce (if (cl:= (sgn number) (sgn divisor)) 0 -0.0)
+                                          type)
+                                  remainder)
+                          (values (flonum-from-rational q type) remainder))))))))
   (define fceiling cl:ceiling)
   (define ffloor cl:floor)
   (define fround cl:round)
@@ -206,47 +168,6 @@
              (cl:expt (rational base) power)
              (pick-result-format base power))))))
 
-(defun %unary-truncate (number)
-  (typecase number
-    (integer number)
-    (ratio (values (truncate (numerator number) (denominator number))))
-    ((or single-float double-float #+long-float long-float)
-     (error "Unimplemented."))))
-
-(defun %unary-ftruncate (number)
-  (typecase number
-    (integer number)
-    (ratio (values (ftruncate (numerator number) (denominator number))))
-    ((or single-float double-float #+long-float long-float)
-     (error "Unimplemented."))))
-
-(defun %unary-round (number)
-  (typecase number
-    (integer number)
-    (ratio (values (round (numerator number) (denominator number))))
-    ((or single-float double-float #+long-float long-float)
-     (error "Unimplemented."))))
-
-(defun %unary-fround (number)
-  (typecase number
-    (integer number)
-    (ratio (values (fround (numerator number) (denominator number))))
-    ((or single-float double-float #+long-float long-float)
-     (error "Unimplemented."))))
-
-(defun scale-float (f ex)
-  (validate-args f)
-  (with-memoized-math-op (scale-float (list f ex))
-    (flonum-from-rational (cl:* (rational f) (expt 2 ex)) (type-of f))))
-
-(defun scale-single-float (f ex)
-  (validate-args f)
-  (scale-float f ex))
-
-(defun scale-double-float (f ex)
-  (validate-args f)
-  (scale-float f ex))
-
 ;;; Four possible return values.  NIL if the numbers (rationals or
 ;;; flonums) are incomparable (either is a NaN).  Otherwise: -1, 0, 1
 ;;; if A is less than, equal to or greater than B.
@@ -328,7 +249,7 @@
                ((eql format 'rational) (cl:+ x y))
                ((and (flonum-minus-zero-p x)
                      (flonum-minus-zero-p y))
-                (coerce $-0.0 format))
+                (coerce -0.0 format))
                (t (flonum-from-rational (cl:+ (rational x) (rational y)) format))))))
     (if (every #'rationalp args)
         (apply #'cl:+ args)
@@ -347,7 +268,7 @@
                ((eql format 'rational) (cl:- x y))
                ((and (flonum-minus-zero-p x)
                      (and (zerop y) (not (flonum-minus-zero-p y))))
-                (coerce $-0.0 format))
+                (coerce -0.0 format))
                (t (flonum-from-rational (cl:- (rational x) (rational y)) format))))))
     (if (every #'rationalp args)
         (apply #'cl:- args)
@@ -361,9 +282,17 @@
            (let ((format (pick-result-format x y)))
              (cond
                ((eql format 'rational) (cl:* x y))
+               ((or (and (floatp x) (float-infinity-p x))
+                    (and (floatp y) (float-infinity-p y)))
+                (when (or (zerop x) (zerop y))
+                  (error "Can't multiply infinity with 0."))
+                (coerce (if (cl:= (sgn x) (sgn y))
+                            single-float-positive-infinity
+                            single-float-negative-infinity)
+                        format))
                ((or (flonum-minus-zero-p x)
                     (flonum-minus-zero-p y))
-                (coerce (if (cl:= (sgn x) (sgn y)) 0 $-0.0) format))
+                (coerce (if (cl:= (sgn x) (sgn y)) 0 -0.0) format))
                (t (flonum-from-rational (cl:* (rational x) (rational y)) format))))))
     (if (every #'rationalp args)
         (apply #'cl:* args)
@@ -375,9 +304,7 @@
            (cond
              ((rationalp x) (cl:/ x))
              ((zerop x)
-              (if (cl:= (float-sign-bit x) 1)
-                  (coerce single-float-negative-infinity (type-of x))
-                  (coerce single-float-positive-infinity (type-of x))))
+              (float-sign x single-float-positive-infinity))
              (t (flonum-from-rational (cl:/ (rational x)) (type-of x)))))
          (two-arg-/ (x y)
            (let ((format (pick-result-format x y)))
@@ -388,7 +315,7 @@
                ((zerop y)
                 (error "can't represent Inf for (/ x 0)"))
                ((zerop x)
-                (coerce (if (cl:= (sgn x) (sgn y)) 0 $-0.0) format))
+                (coerce (if (cl:= (sgn x) (sgn y)) 0 -0.0) format))
                (t (flonum-from-rational (cl:/ (rational x) (rational y)) format))))))
     (if (every #'rationalp args)
         (apply #'cl:/ args)
@@ -410,7 +337,7 @@
 
 (defun sb-xc:sqrt (arg)
   (let ((format (if (rationalp arg) 'single-float (type-of arg))))
-    (with-memoized-math-op (sqrt (list arg))
+    (with-memoized-math-op (sqrt arg)
       (flonum-from-rational (%sqrt (rational arg)) format))))
 
 ;;; There seems to be no portable way to mask float traps, so right
@@ -425,8 +352,8 @@
 (defun realpart (x) (if (realp x) x (complexnum-real x)))
 (defun imagpart (x)
   (cond ((rationalp x) 0)
-        ((single-float-p x) $0f0)
-        ((double-float-p x) $0d0)
+        ((single-float-p x) 0f0)
+        ((double-float-p x) 0d0)
         (t (complexnum-imag x))))
 
 (defun sb-vm::sign-extend (x size)
@@ -434,7 +361,7 @@
 
 ;;; PI is needed in order to build the cross-compiler mainly so that vm-fndb
 ;;; can define bounds on irrational functions.
-(defconstant pi $3.14159265358979323846264338327950288419716939937511L0)
+(defconstant pi 3.14159265358979323846264338327950288419716939937511L0)
 
 (macrolet ((def (name lambda-list)
              `(defun ,(intern (string name) "SB-XC") ,lambda-list
@@ -459,52 +386,52 @@
   (if number2p
       (with-memoized-math-op (atan (list number1 number2))
         (error "Unimplemented."))
-      (with-memoized-math-op (atan (list number1))
-        (if (eql number1 $1.4916681462400417d-154)
+      (with-memoized-math-op (atan number1)
+        (if (eql number1 1.4916681462400417d-154)
             number1
             (error "Unimplemented.")))))
 
 (defun cosh (number)
-  (with-memoized-math-op (cosh (list number))
+  (with-memoized-math-op (cosh number)
     (case number
-      ((0 $0f0) $1f0)
-      ($0d0 $1d0)
+      ((0 0f0) 1f0)
+      (0d0 1d0)
       (t (error "Unimplemented.")))))
 
 (defun log (number &optional (base nil base-p))
   (validate-args number base)
-  (with-memoized-math-op (log (cons number (if base-p (list base))))
+  (with-memoized-math-op (log (if base-p (list number base) number))
     (let ((format (pick-float-result-format number (if base-p base 0))))
       (if (zerop number)
           (coerce single-float-negative-infinity format)
           (case base
             ((nil)
-             (let ((table '((1 . $0f0)
-                            (10 . $2.3025851f0)
-                            (#x1fffffff . $20.101269f0)
-                            (#x20000000 . $20.101269f0)
-                            (#x20000001 . $20.101269f0)
-                            (#xfffffffffffffff . $41.58883f0)
-                            (#x1000000000000000 . $41.58883f0)
-                            (#x1000000000000001 . $41.58883f0)
-                            (#x3fffffffffffffff . $42.975124f0)
-                            (#x4000000000000000 . $42.975124f0)
-                            (#x4000000000000001 . $42.975124f0)
-                            ($0.9999999999999999d0 . $-1.1102230246251565d-16)
-                            ($1d0 . $0d0)
-                            ($2d0 . $0.6931471805599453d0)
-                            ($2.718281828459045d0 . $1d0)
-                            ($5.36870911d8 . $20.10126823437577d0)
-                            ($5.36870912d8 . $20.101268236238415d0)
-                            ($2.147483647d9 . $21.487562596892644d0)
-                            ($2.147483648d9 . $21.487562597358306d0)
-                            ($1.152921504606847d18 . $41.58883083359672d0)
-                            ($4.611686018427388d18 . $42.97512519471661d0)
-                            ($9.223372036854776d18 . $43.66827237527655d0))))
+             (let ((table '((1 . 0f0)
+                            (10 . 2.3025851f0)
+                            (#x1fffffff . 20.101269f0)
+                            (#x20000000 . 20.101269f0)
+                            (#x20000001 . 20.101269f0)
+                            (#xfffffffffffffff . 41.58883f0)
+                            (#x1000000000000000 . 41.58883f0)
+                            (#x1000000000000001 . 41.58883f0)
+                            (#x3fffffffffffffff . 42.975124f0)
+                            (#x4000000000000000 . 42.975124f0)
+                            (#x4000000000000001 . 42.975124f0)
+                            (0.9999999999999999d0 . -1.1102230246251565d-16)
+                            (1d0 . 0d0)
+                            (2d0 . 0.6931471805599453d0)
+                            (2.718281828459045d0 . 1d0)
+                            (5.36870911d8 . 20.10126823437577d0)
+                            (5.36870912d8 . 20.101268236238415d0)
+                            (2.147483647d9 . 21.487562596892644d0)
+                            (2.147483648d9 . 21.487562597358306d0)
+                            (1.152921504606847d18 . 41.58883083359672d0)
+                            (4.611686018427388d18 . 42.97512519471661d0)
+                            (9.223372036854776d18 . 43.66827237527655d0))))
                (or (cdr (assoc number table))
                    (error "missing entry for (LOG ~A)" number))))
-            ((10 $10f0 $10d0)
-             (let ((table '(($2d0 . $0.3010299956639812d0))))
+            ((10 10f0 10d0)
+             (let ((table '((2d0 . 0.3010299956639812d0))))
                (or (cdr (assoc number table))
                    (error "missing entry for (LOG ~A 10)" number))))
             (t (error "missing entries for (LOG ~A ~A)" number base)))))))
@@ -581,54 +508,54 @@
                  table)))))
 
 ;;; Perform some simple checks
-(assert (not (eq $-0.0f0 $-0.0d0)))
-(assert (not (eq single-float-negative-infinity $0f0)))
+(assert (not (eq -0.0f0 -0.0d0)))
+(assert (not (eq single-float-negative-infinity 0f0)))
 (dolist (format '(single-float double-float))
   (assert (zerop (coerce 0 format)))
-  (assert (zerop (coerce $-0.0 format)))
+  (assert (zerop (coerce -0.0 format)))
   (assert (float-infinity-p (coerce single-float-positive-infinity format)))
   (assert (float-infinity-or-nan-p (coerce single-float-positive-infinity format)))
   (assert (not (float-nan-p (coerce single-float-positive-infinity format))))
   (assert (float-infinity-p (coerce single-float-negative-infinity format)))
   (assert (float-infinity-or-nan-p (coerce single-float-negative-infinity format)))
   (assert (not (float-nan-p (coerce single-float-negative-infinity format))))
-  (assert (eq (coerce $-0.0 format) (coerce $-0.0 format)))
+  (assert (eq (coerce -0.0 format) (coerce -0.0 format)))
   (assert (eq (coerce single-float-positive-infinity format)
               (coerce single-float-positive-infinity format)))
   (assert (eq (coerce single-float-negative-infinity format)
               (coerce single-float-negative-infinity format)))
-  (assert (eq (sb-xc:+ (coerce $-0.0 format) (coerce 0 format)) (coerce 0 format)))
-  (assert (eq (sb-xc:+ (coerce 0 format) (coerce $-0.0 format)) (coerce 0 format)))
-  (assert (eq (sb-xc:+ (coerce $-0.0 format) (coerce $-0.0 format)) (coerce $-0.0 format)))
-  (assert (eq (sb-xc:- (coerce 0 format)) (coerce $-0.0 format)))
-  (assert (eq (sb-xc:- (coerce $-0.0 format)) (coerce 0 format)))
+  (assert (eq (sb-xc:+ (coerce -0.0 format) (coerce 0 format)) (coerce 0 format)))
+  (assert (eq (sb-xc:+ (coerce 0 format) (coerce -0.0 format)) (coerce 0 format)))
+  (assert (eq (sb-xc:+ (coerce -0.0 format) (coerce -0.0 format)) (coerce -0.0 format)))
+  (assert (eq (sb-xc:- (coerce 0 format)) (coerce -0.0 format)))
+  (assert (eq (sb-xc:- (coerce -0.0 format)) (coerce 0 format)))
   (assert (eq (coerce single-float-positive-infinity format)
               (sb-xc:- (coerce single-float-negative-infinity format))))
   (assert (eq (coerce single-float-negative-infinity format)
               (sb-xc:- (coerce single-float-positive-infinity format))))
   (assert (eq (sb-xc:- (coerce 0 format) (coerce 0 format)) (coerce 0 format)))
-  (assert (eq (sb-xc:- (coerce 0 format) (coerce $-0.0 format)) (coerce 0 format)))
-  (assert (eq (sb-xc:- (coerce $-0.0 format) (coerce 0 format)) (coerce $-0.0 format)))
-  (assert (eq (sb-xc:- (coerce $-0.0 format) (coerce $-0.0 format)) (coerce 0 format)))
+  (assert (eq (sb-xc:- (coerce 0 format) (coerce -0.0 format)) (coerce 0 format)))
+  (assert (eq (sb-xc:- (coerce -0.0 format) (coerce 0 format)) (coerce -0.0 format)))
+  (assert (eq (sb-xc:- (coerce -0.0 format) (coerce -0.0 format)) (coerce 0 format)))
   (assert (eq (sb-xc:* (coerce 0 format) (coerce 0 format)) (coerce 0 format)))
-  (assert (eq (sb-xc:* (coerce $-0.0 format) (coerce 0 format)) (coerce $-0.0 format)))
-  (assert (eq (sb-xc:* (coerce 0 format) (coerce $-0.0 format)) (coerce $-0.0 format)))
-  (assert (eq (sb-xc:* (coerce $-0.0 format) (coerce $-0.0 format)) (coerce 0 format)))
-  (assert (eq (sb-xc:/ (coerce $-0.0 format) (coerce -1 format)) (coerce 0 format)))
-  (assert (eq (sb-xc:/ (coerce $-0.0 format) (coerce 1 format)) (coerce $-0.0 format)))
-  (assert (eq (sb-xc:/ (coerce 0 format) (coerce -1 format)) (coerce $-0.0 format)))
+  (assert (eq (sb-xc:* (coerce -0.0 format) (coerce 0 format)) (coerce -0.0 format)))
+  (assert (eq (sb-xc:* (coerce 0 format) (coerce -0.0 format)) (coerce -0.0 format)))
+  (assert (eq (sb-xc:* (coerce -0.0 format) (coerce -0.0 format)) (coerce 0 format)))
+  (assert (eq (sb-xc:/ (coerce -0.0 format) (coerce -1 format)) (coerce 0 format)))
+  (assert (eq (sb-xc:/ (coerce -0.0 format) (coerce 1 format)) (coerce -0.0 format)))
+  (assert (eq (sb-xc:/ (coerce 0 format) (coerce -1 format)) (coerce -0.0 format)))
   (assert (eq (sb-xc:/ (coerce 0 format) (coerce 1 format)) (coerce 0 format)))
-  (assert (eq (sb-xc:fceiling -1/2) $-0.0f0))
-  (assert (eq (sb-xc:fceiling (coerce -1/2 format)) (coerce $-0.0 format)))
+  (assert (eq (sb-xc:fceiling -1/2) -0.0f0))
+  (assert (eq (sb-xc:fceiling (coerce -1/2 format)) (coerce -0.0 format)))
   (assert (eq (sb-xc:ffloor -1/2) (coerce -1 'single-float)))
   (assert (eq (sb-xc:ffloor (coerce -1/2 format)) (coerce -1 format)))
-  (assert (eq (sb-xc:ftruncate -1/2) $-0.0f0))
-  (assert (eq (sb-xc:ftruncate (coerce -1/2 format)) (coerce $-0.0 format)))
-  (assert (eq (sb-xc:fround -1/2) $-0.0f0))
-  (assert (eq (sb-xc:fround (coerce -1/2 format)) (coerce $-0.0 format)))
-  (assert (equal (multiple-value-list (sb-xc:integer-decode-float $1.0f0))
+  (assert (eq (sb-xc:ftruncate -1/2) -0.0f0))
+  (assert (eq (sb-xc:ftruncate (coerce -1/2 format)) (coerce -0.0 format)))
+  (assert (eq (sb-xc:fround -1/2) -0.0f0))
+  (assert (eq (sb-xc:fround (coerce -1/2 format)) (coerce -0.0 format)))
+  (assert (equal (multiple-value-list (sb-xc:integer-decode-float 1.0f0))
                  '(8388608 -23 1)))
-  (assert (equal (multiple-value-list (sb-xc:integer-decode-float $1.0d0))
+  (assert (equal (multiple-value-list (sb-xc:integer-decode-float 1.0d0))
                  '(4503599627370496 -52 1)))
   (let ((*break-on-signals* nil))
   (flet ((assert-not-number (x)
