@@ -1473,7 +1473,7 @@
                   (:declare (type (integer 0 63) sh) (type (or (integer 0 63) fixup) m))
                   (:printer md-form ((op 30) (subop ,op) (rc ,rc)))
                   (:emitter
-                   (when (and (fixup-p m) (eq (fixup-flavor m) :gc-barrier))
+                   (when (and (fixup-p m) (eq (fixup-flavor m) :card-table-index-mask))
                      (note-fixup segment :rldic-m m)
                      (setq m 0))
                    (emit-md-form-inst segment 30
@@ -2232,6 +2232,7 @@
   (:delay 0)
   (:emitter
    (etypecase word
+     #-64-bit
      (fixup
       (note-fixup segment :absolute word)
       (emit-word segment 0))
@@ -2243,6 +2244,7 @@
   (:delay 0)
   (:emitter
    (etypecase dword
+     #+64-bit
      (fixup
       (note-fixup segment :absolute dword)
       (emit-dword segment 0))
@@ -2394,23 +2396,20 @@
                    `(.byte ,@bytes)))))))
 
 (defun sb-vm:fixup-code-object (code offset value kind flavor)
-  (declare (type index offset) (ignore flavor))
+  (declare (type index offset))
   (unless (zerop (rem offset sb-assem:+inst-alignment-bytes+))
     (error "Unaligned instruction?  offset=#x~X." offset))
   (let ((sap (code-instructions code)))
     (ecase kind
       (:absolute
        ;; There is an implicit addend currently stored in the fixup location.
-       (incf (sap-ref-32 sap offset) value))
-      (:absolute64
-       (incf (sap-ref-64 sap offset) value))
+       (incf (sap-ref-word sap offset) value))
       (:layout-id
        (aver (zerop (sap-ref-32 sap offset)))
        (setf (signed-sap-ref-32 sap offset) (the layout-id value)))
       (:rldic-m ; This is the M (mask) immediate operand to RLDIC{L,R} which
        ;; appears in (byte 6 5) of the instruction. See EMIT-MD-FORM-INST.
-       (setf (ldb (byte 6 5) (sap-ref-32 sap offset)) (encode-mask6 (- 64 value)))
-       (return-from fixup-code-object :immediate))
+       (setf (ldb (byte 6 5) (sap-ref-32 sap offset)) (encode-mask6 (- 64 value))))
       (:b
        (error "Can't deal with CALL fixups, yet."))
       (:ba
@@ -2428,6 +2427,8 @@
           (setf (ldb (byte 16 0) (sap-ref-32 sap offset))
                  (if (logbitp 15 l) (ldb (byte 16 0) (1+ h)) h))))
       (:l
+       (when (eq flavor :assembly-routine)
+         (decf value nil-value))
        (setf (ldb (byte 16 0) (sap-ref-32 sap offset))
              (ldb (byte 16 0) value)))))
   nil)
@@ -2437,7 +2438,9 @@
    ;; No backpatch is needed to compute the offset into the code header
    ;; because COMPONENT-HEADER-LENGTH is known at this point.
    (let ((offset (+ (component-header-length)
-                    n-word-bytes ; skip over jump table word
+                    ;; skip over jump table word and entries
+                    (* (1+ (component-n-jump-table-entries))
+                       n-word-bytes)
                     mark-index
                     (- code-tn-lowtag))))
      (inst* segment 'stb sb-vm::null-tn sb-vm::code-tn
