@@ -35,7 +35,7 @@
 #include "thread.h"
 #include "runtime.h"
 #include "genesis/static-symbols.h"
-#include "genesis/fdefn.h"
+#include "genesis/symbol.h"
 
 #include <sys/types.h>
 #include <signal.h>
